#include <map>

#include "gfxapi.h"

#include "shader_manager.h"
#include "opengl/shader-gl.h"

ShaderManager::ShaderManager() {}
ShaderManager::~ShaderManager() {}

void ShaderManager::add_shader(Shader *sdr, const char *name)
{
	shaders[name] = sdr;
}

Shader *ShaderManager::load_shader(const char *name, SType type)
{
	std::map<std::string, Shader *>::const_iterator it;
	it = shaders.find(name);

	if(it != shaders.end()) {
		return it->second;
	}

	Shader *sdr = gfx_create_shader();

	std::string fname = path.empty() ? std::string(name) : path + "/" + std::string(name);

	if(!sdr->load(fname.c_str(), type)) {
		fprintf(stderr, "Failed to load shader: %s.\n", fname.c_str());
		delete sdr;
		return 0;
	}

	add_shader(sdr, name);
	return sdr;
}

void ShaderManager::delete_shaders()
{
	std::map<std::string, Shader *>::iterator it;
	for(it=shaders.begin(); it != shaders.end(); it++) {
		delete it->second;
	}
}

ShaderProgram *ShaderManager::create_shader_program(const char *vname, const char *fname)
{
	path = std::string(gfx_get_shader_path());

	Shader *vsdr = load_shader(vname, SDR_VERTEX);
	if(!vsdr)
		return 0;

	Shader *fsdr = load_shader(fname, SDR_FRAGMENT);
	if(!fsdr)
		return 0;

	ShaderProgram *sprog = gfx_create_shader_program();

	if(!sprog->create()) {
		return 0;
	}

	sprog->attach_shader(vsdr);
	sprog->attach_shader(fsdr);

	if(!sprog->link()) {
		delete sprog;
		return 0;
	}

	return sprog;
}

void ShaderManager::set_path(const char *path)
{
	if(!path)
		this->path.clear();
	else
		this->path = std::string(path);
}